<?php
/* --------------------------------------------------------------
 GoogleOAuthController.inc.php 2017-12-20
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2017 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

class GoogleOAuthController extends AdminHttpViewController
{
    /**
     * @var string
     */
    protected $jsBaseUrl;
    
    /**
     * @var string
     */
    protected $stylesBaseUrl;
    
    /**
     * @var string
     */
    protected $templatesBaseUrl;
    
    /**
     * @var \LanguageTextManager
     */
    protected $languageTextManager;
    
    /**
     * @var \CI_DB_query_builder
     */
    protected $db;
    
    
    /**
     * Initialize the controller.
     */
    public function init()
    {
        $this->jsBaseUrl           = '../GXModules/Gambio/GoogleAdWords/Build/Admin/Javascript';
        $this->stylesBaseUrl       = '../GXModules/Gambio/GoogleAdWords/Build/Admin/Styles';
        $this->templatesBaseUrl    = DIR_FS_CATALOG . 'GXModules/Gambio/GoogleOAuth/Admin/Html';
        $this->languageTextManager = MainFactory::create('LanguageTextManager', 'google_adwords',
            $_SESSION['languages_id']);
        $this->db                  = StaticGXCoreLoader::getDatabaseQueryBuilder();
    }
    
    
    public function actionDefault()
    {
        $title    = new NonEmptyStringType($this->languageTextManager->get_text('page_title'));
        $template = new ExistingFile(new NonEmptyStringType($this->templatesBaseUrl . '/connect_account.html'));
        
        $data = MainFactory::create('KeyValueCollection', [
            'iFrameUrl' => gm_get_conf('GOOGLE_ADWORDS_APP_URL') . '#connect',
            'origin'    => HTTP_SERVER . DIR_WS_CATALOG,
            'error'     => (int)$this->_getQueryParameter('error'),
            'language'  => $_SESSION['language_code'],
        ]);
        
        $assets = MainFactory::create('AssetCollection', [
            MainFactory::create('Asset', $this->stylesBaseUrl . '/partials/connect_account.css'),
            MainFactory::create('Asset', $this->jsBaseUrl . '/vendor/iframe_resizer.js'),
            MainFactory::create('Asset', $this->jsBaseUrl . '/extensions/resize_iframe.js'),
        ]);
        
        return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data, $assets);
    }
    
    
    public function actionReset()
    {
        $db = StaticGXCoreLoader::getDatabaseQueryBuilder();
        $db->truncate('google_adwords_client_customers');
        
        gm_set_conf('GOOGLE_ADWORDS_CONNECTION_STATUS', '');
        gm_set_conf('GOOGLE_ADWORDS_REFRESH_TOKEN', '');
        
        return new RedirectHttpControllerResponse('./admin.php?do=GoogleOAuth');
    }
}