<?php
/* --------------------------------------------------------------
   MoneyOrderPlusHubController.inc.php 2018-04-17
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class MoneyOrderPlusHubController
 *
 * @category   System
 * @package    AdminHttpViewControllers
 * @extends    AdminHttpViewController
 */
class MoneyOrderPlusHubController extends AdminHttpViewController
{
	/**
	 * @var string
	 */
	protected $configurationKey = 'GAMBIO_HUB_MONEY_ORDER_PLUS_HUB_USER_INTERFACE_URL';
	
	/**
	 * @var string
	 */
	protected $scriptsBaseUrl;
	
	/**
	 * @var string
	 */
	protected $stylesBaseUrl;
	
	/**
	 * @var string
	 */
	protected $minPostfix;
	
	/**
	 * @var LanguageTextManager
	 */
	protected $languageTextManager;
	
	/**
	 * @var HubAssetHelper
	 */
	protected $hubAssetHelper;
	
	
	/**
	 * Initialize the controller.
	 */
	public function init()
	{
		if(isset($_SESSION['coo_page_token']) == false)
		{
			throw new RuntimeException('Page Token Generator not found.'); // CSRF Protection
		}
		
		$installedVersion     = gm_get_conf('INSTALLED_VERSION');
		$this->hubAssetHelper = MainFactory::create('HubAssetHelper', $installedVersion);
		
		$this->scriptsBaseUrl      = DIR_WS_CATALOG . $this->hubAssetHelper->getScriptsBaseUrl();
		$this->stylesBaseUrl       = DIR_WS_CATALOG . $this->hubAssetHelper->getStylesBaseUrl();
		$this->minPostfix          = file_exists(DIR_FS_CATALOG . '.dev-environment') ? '' : '.min';
		$this->languageTextManager = MainFactory::create('LanguageTextManager', 'gambio_hub_money_order_plus_hub',
		                                                 $_SESSION['languages_id']);
	}
	
	
	/**
	 * Renders the money order interface page.
	 *
	 * @return HttpControllerResponseInterface
	 */
	public function actionDefault()
	{
		$title = new NonEmptyStringType($this->languageTextManager->get_text('PAGE_TITLE',
		                                                                     'gambio_hub_money_order_plus_hub'));
		
		$template = new ExistingFile(new NonEmptyStringType(DIR_FS_CATALOG
		                                                    . $this->hubAssetHelper->getTemplatesBasePath()
		                                                    . '/money_order_plus.html'));
		
		$userInterfaceUrl = gm_get_conf($this->configurationKey);
		
		if(empty($userInterfaceUrl))
		{
			return MainFactory::create(RedirectHttpControllerResponse::class,
			                           DIR_WS_ADMIN . 'admin.php?do=HubConfiguration/PaymentMethods');
		}
		
		$data = MainFactory::create('KeyValueCollection', [
			'iframe_source_url' => $userInterfaceUrl . '?language=' . $_SESSION['language_code']
		]);
		
		$assetsArray = [
			MainFactory::create('Asset', $this->scriptsBaseUrl . '/vendor/promise' . $this->minPostfix . '.js'),
			MainFactory::create('Asset', $this->scriptsBaseUrl . '/vendor/fetch' . $this->minPostfix . '.js'),
			MainFactory::create('Asset', $this->scriptsBaseUrl . '/vendor/iframe_resizer' . $this->minPostfix . '.js'),
			MainFactory::create('Asset', $this->stylesBaseUrl . '/gambio_hub' . $this->minPostfix . '.css')
		];
		
		$assets = MainFactory::create('AssetCollection', $assetsArray);
		
		return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data, $assets);
	}
}