/* --------------------------------------------------------------
 delete.js 2016-10-17
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2016 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

/**
 * Delete Order Modal Controller
 */
gx.controllers.module('delete', ['modal'], function(data) {
	
	'use strict';
	
	// ------------------------------------------------------------------------
	// VARIABLES
	// ------------------------------------------------------------------------
	
	/**
	 * Module Selector
	 *
	 * @type {jQuery}
	 */
	const $this = $(this);
	
	/**
	 * Module Instance
	 *
	 * @type {Object}
	 */
	const module = {
		bindings: {
			selectedOrders: $this.find('.selected-orders'),
			reStock: $this.find('.re-stock'),
			reShip: $this.find('.re-ship'),
			reActivate: $this.find('.re-activate')
		}
	};
	
	// ------------------------------------------------------------------------
	// FUNCTIONS
	// ------------------------------------------------------------------------
	
	/**
	 * Send the modal data to the form through an AJAX call.
	 *
	 * @param {jQuery.Event} event
	 */
	function _onDeleteClick(event) {
		const url = jse.core.config.get('appUrl') + '/admin/admin.php?do=OrdersModalsAjax/DeleteOrder';
		const data = {
			selectedOrders: module.bindings.selectedOrders.get().split(', '),
			reStock: module.bindings.reStock.get(),
			reShip: module.bindings.reShip.get(),
			reActivate: module.bindings.reActivate.get(),
			pageToken: jse.core.config.get('pageToken')
		};
		const $deleteButton = $(event.target);
		
		$deleteButton.addClass('disabled').prop('disabled', true);
		
		$.ajax({
			url,
			data,
			method: 'POST',
			dataType: 'json'
		})
			.done(function(response) {
				jse.libs.info_box.addSuccessMessage(
					jse.core.lang.translate('DELETE_ORDERS_SUCCESS', 'admin_orders'));
				$('.orders .table-main').DataTable().ajax.reload(null, false);
				$('.orders .table-main').orders_overview_filter('reload');
			})
			.fail(function(jqxhr, textStatus, errorThrown) {
				jse.libs.modal.showMessage(jse.core.lang.translate('error', 'messages'),
					jse.core.lang.translate('DELETE_ORDERS_ERROR', 'admin_orders'));
			})
			.always(function() {
				$this.modal('hide');
				$deleteButton.removeClass('disabled').prop('disabled', false);
			});
	}
	
	// ------------------------------------------------------------------------
	// INITIALIZATION
	// ------------------------------------------------------------------------
	
	module.init = function(done) {
		$this.on('click', '.btn.delete', _onDeleteClick);
		done();
	};
	
	return module;
});